<?php
// +-----------------------------------------------------------------+
// |                   PhreeBooks Open Source ERP                    |
// +-----------------------------------------------------------------+
// | Copyright (c) 2008, 2009, 2010 PhreeSoft, LLC                   |
// | http://www.PhreeSoft.com                                        |
// +-----------------------------------------------------------------+
// | This program is free software: you can redistribute it and/or   |
// | modify it under the terms of the GNU General Public License as  |
// | published by the Free Software Foundation, either version 3 of  |
// | the License, or any later version.                              |
// |                                                                 |
// | This program is distributed in the hope that it will be useful, |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of  |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
// | GNU General Public License for more details.                    |
// |                                                                 |
// | The license that is bundled with this package is located in the |
// | file: /doc/manual/ch01-Introduction/license.html.               |
// | If not, see http://www.gnu.org/licenses/                        |
// +-----------------------------------------------------------------+
//  Path: /modules/services/payment/modules/nova_xml.php
//

/**
 * Nova Network Payment Module (viaKLIX version)
 * You must have SSL active on your server to be compliant with merchant TOS
 *
 */
class nova_xml {
  /**
   * $code determines the internal 'code' name used to designate "this" payment module
   *
   * @var string
   */
  var $code;
  /**
   * $title is the displayed name for this payment method
   *
   * @var string
   */
  var $title;
  /**
   * $description is a soft name for this payment method
   *
   * @var string
   */
  var $description;
  /**
   * $enabled determines whether this module shows or not... in catalog.
   *
   * @var boolean
   */
  var $enabled;
  /**
     * Constructor
     *
     * @return nova
     */
  function nova_xml() {
    global $order;
    $this->code = 'nova_xml';
    $this->title = MODULE_PAYMENT_NOVA_XML_TEXT_TITLE; // Payment module title in Catalog
    $this->description = MODULE_PAYMENT_NOVA_XML_TEXT_DESCRIPTION; // Descriptive Info about module in Admin
    $this->enabled = ((MODULE_PAYMENT_NOVA_XML_STATUS == 'True') ? true : false); // Whether the module is installed or not
    $this->sort_order = MODULE_PAYMENT_NOVA_XML_SORT_ORDER; // Sort Order of this payment option on the customer payment page
	$this->enable_encryption = 1; // set to field position of credit card to create hint, false to turn off encryption

    if ((int)MODULE_PAYMENT_NOVA_XML_ORDER_STATUS_ID > 0) {
      $this->order_status = MODULE_PAYMENT_NOVA_XML_ORDER_STATUS_ID;
    }

	// save the information
	// Card numbers are not saved, instead keep the first and last four digits and fill middle with *'s
	$card_number = trim($_POST['nova_xml_field_1']);
	$card_number = substr($card_number, 0, 4) . '********' . substr($card_number, -4);
	$this->payment_fields = implode(':', array($_POST['nova_xml_field_0'], $card_number, $_POST['nova_xml_field_2'], $_POST['nova_xml_field_3'], $_POST['nova_xml_field_4']));

	$this->avs_codes = array(
		'A' => 'Address matches - Postal Code does not match.',
		'B' => 'Street address match, Postal code in wrong format. (International issuer)',
		'C' => 'Street address and postal code in wrong formats.',
		'D' => 'Street address and postal code match. (international issuer)',
		'E' => 'AVS Error.',
		'G' => 'Service not supported by non-US issuer.',
		'I' => 'Address information not verified by international issuer.',
		'M' => 'Street address and Postal code match. (international issuer)',
		'N' => 'No match on address (street) or postal code.',
		'O' => 'No response sent.',
		'P' => 'Postal code matches, street address not verified due to incompatible formats.',
		'R' => 'Retry, system unavailable or timed out.',
		'S' => 'Service not supported by issuer.',
		'U' => 'Address information is unavailable.',
		'W' => '9 digit postal code matches, address (street) does not match.',
		'X' => 'Exact AVS match.',
		'Y' => 'Address (street) and 5 digit postal code match.',
		'Z' => '5 digit postal code matches, address (street) does not match.'
	);

	$this->cvv_codes = array(
		'M' => 'CVV2 match',
		'N' => 'CVV2 No match',
		'P' => 'Not Processed',
		'S' => 'Issuer indicates that CVV2 data should be present on the card, but the merchant has indicated that the CVV2 data is not present on the card.',
		'U' => 'Issuer has not certified for CVV2 or issuer has not provided Visa with the CVV2 encryption keys.'
	);
  }
  /**
   * JS validation which does error-checking of data-entry if this module is selected for use
   * (Number, Owner, and CVV Lengths)
   *
   * @return string
   */
  function javascript_validation() {
    $js = '  if (payment_method == "' . $this->code . '") {' . "\n" .
    '    var cc_owner = document.getElementById("nova_xml_field_0").value;' . "\n" .
    '    var cc_number = document.getElementById("nova_xml_field_1").value;' . "\n";
    if (MODULE_PAYMENT_NOVA_XML_USE_CVV == 'True')  {
      $js .= '    var cc_cvv = document.getElementById("nova_xml_field_4").value;' . "\n";
    }
    $js .= '    if (cc_owner == "" || cc_owner.length < ' . CC_OWNER_MIN_LENGTH . ') {' . "\n" .
    '      error_message = error_message + "' . MODULE_PAYMENT_NOVA_XML_TEXT_JS_CC_OWNER . '";' . "\n" .
    '      error = 1;' . "\n" .
    '    }' . "\n" .
    '    if (cc_number == "" || cc_number.length < ' . CC_NUMBER_MIN_LENGTH . ') {' . "\n" .
    '      error_message = error_message + "' . MODULE_PAYMENT_NOVA_XML_TEXT_JS_CC_NUMBER . '";' . "\n" .
    '      error = 1;' . "\n" .
    '    }' . "\n";
    if (MODULE_PAYMENT_NOVA_XML_USE_CVV == 'True')  {
      $js .= '    if (cc_cvv == "" || cc_cvv.length < "3" || cc_cvv.length > "4") {' . "\n".
      '      error_message = error_message + "' . MODULE_PAYMENT_NOVA_XML_TEXT_JS_CC_CVV . '";' . "\n" .
      '      error = 1;' . "\n" .
      '    }' . "\n" ;
    }
    $js .= '  }' . "\n";

    return $js;
  }
  /**
   * Display Credit Card Information Submission Fields on the Checkout Payment Page
   *
   * @return array
   */
  function selection() {
    global $order;

    for ($i=1; $i<13; $i++) {
      $expires_month[] = array('id' => sprintf('%02d', $i), 'text' => strftime('%B',mktime(0,0,0,$i,1,2000)));
    }

    $today = getdate();
    for ($i = $today['year']; $i < $today['year'] + 10; $i++) {
      $expires_year[] = array('id' => strftime('%y',mktime(0,0,0,1,1,$i)), 'text' => strftime('%Y',mktime(0,0,0,1,1,$i)));
    }
	$selection = array(
	   'id' => $this->code,
	   'module' => MODULE_PAYMENT_NOVA_XML_TEXT_CATALOG_TITLE,
	   'fields' => array(array('title' => MODULE_PAYMENT_NOVA_XML_TEXT_CREDIT_CARD_OWNER,
					   'field' => html_input_field('nova_xml_field_0', $order->nova_xml_field_0)),
				   array('title' => MODULE_PAYMENT_NOVA_XML_TEXT_CREDIT_CARD_NUMBER,
					   'field' => html_input_field('nova_xml_field_1', $order->nova_xml_field_1)),
				   array('title' => MODULE_PAYMENT_NOVA_XML_TEXT_CREDIT_CARD_EXPIRES,
					   'field' => html_pull_down_menu('nova_xml_field_2', $expires_month, $order->nova_xml_field_2) . '&nbsp;' . html_pull_down_menu('nova_xml_field_3', $expires_year, $order->nova_xml_field_3)),
				   array('title' => MODULE_PAYMENT_NOVA_XML_TEXT_CVV,
					   'field' => html_input_field('nova_xml_field_4', $order->nova_xml_field_4, 'size="4" maxlength="4"'))
		 ));
    return $selection;
  }
  /**
   * Evaluates the Credit Card Type for acceptance and the validity of the Credit Card Number & Expiration Date
   *
   */
  function pre_confirmation_check() {
    global $_POST, $messageStack;

	// if the card number has the blanked out middle number fields, it has been processed, show message that 
	// the charges were not processed through the merchant gateway and continue posting payment.
	if (strpos($_POST['nova_xml_field_1'],'*') !== false) {
    	$messageStack->add(MODULE_PAYMENT_NOVA_XML_NO_DUPS, 'caution');
		return false;
	}

    include_once(DIR_FS_MODULES . 'general/classes/cc_validation.php');

    $cc_validation = new cc_validation();
    $result = $cc_validation->validate($_POST['nova_xml_field_1'], $_POST['nova_xml_field_2'], $_POST['nova_xml_field_3'], $_POST['nova_xml_field_4']);
    $error = '';
    switch ($result) {
      case -1:
      $error = sprintf(TEXT_CCVAL_ERROR_UNKNOWN_CARD, substr($cc_validation->cc_number, 0, 4));
      break;
      case -2:
      case -3:
      case -4:
      $error = TEXT_CCVAL_ERROR_INVALID_DATE;
      break;
      case false:
      $error = TEXT_CCVAL_ERROR_INVALID_NUMBER;
      break;
    }

    if ( ($result == false) || ($result < 1) ) {
      $messageStack->add($error . '<!-- ['.$this->code.'] -->', 'error');
      return true;
    }

    $this->cc_card_type = $cc_validation->cc_type;
    $this->cc_card_number = $cc_validation->cc_number;
    $this->cc_cvv2 = $_POST['nova_xml_field_4'];
    $this->cc_expiry_month = $cc_validation->cc_expiry_month;
    $this->cc_expiry_year = $cc_validation->cc_expiry_year;
	return false;
  }
  /**
   * Store the CC info to the order and process any results that come back from the payment gateway
   *
   */
  function before_process() {
    global $order, $db, $messageStack;

	// if the card number has the blanked out middle number fields, it has been processed, the message that 
	// the charges were not processed were set in pre_confirmation_check, just return to continue without processing.
	if (strpos($_POST['nova_xml_field_1'], '*') !== false) {
		return false;
	}

    $order->info['cc_expires'] = $_POST['nova_xml_field_2'] . $_POST['nova_xml_field_3'];
    $order->info['cc_owner'] = $_POST['nova_xml_field_0'];
	$this->cc_card_owner = $_POST['nova_xml_field_0'];
    $order->info['cc_cvv'] = $_POST['nova_xml_field_4'];

    // Create a string that contains a listing of products ordered for the description field
    $description = $order->description;

    // Populate an array that contains all of the data to be sent to Nova (their xml string is one level)
    $submit_data = array(
		'ssl_transaction_type'   => ((MODULE_PAYMENT_NOVA_XML_AUTHORIZATION_TYPE == 'Authorize') ? 'CCAUTHONLY' : 'CCSALE'),
		'ssl_merchant_id'        => MODULE_PAYMENT_NOVA_XML_MERCHANT_ID, 
		'ssl_pin'                => MODULE_PAYMENT_NOVA_XML_PIN, 
		'ssl_user_id'            => MODULE_PAYMENT_NOVA_XML_USER_ID, 
		'ssl_amount'             => $order->total_amount,
		'ssl_salestax'           => ($order->sales_tax) ? $order->sales_tax : 0,
		'ssl_card_number'        => preg_replace('/ /', '', $_POST['nova_xml_field_1']),
		'ssl_exp_date'           => $order->info['cc_expires'],
		'ssl_cvv2cvc2_indicator' => ($this->cc_cvv2) ? '1' : '9', // if cvv2 exists, present else not present
		'ssl_cvv2cvc2'           => ($this->cc_cvv2) ? $this->cc_cvv2 : '',
		'ssl_description'        => $description,
		'ssl_invoice_number'     => (MODULE_PAYMENT_NOVA_XML_TESTMODE == 'Test' ? 'TEST-' : '') . $order->purchase_invoice_id,
        'ssl_customer_code'      => $order->bill_short_name,
		'ssl_company'            => str_replace('&', '-', $order->bill_primary_name),
		'ssl_first_name'         => $order->bill_first_name, // passed with credit card form
		'ssl_last_name'          => $order->bill_last_name, // passed with credit card form
		'ssl_avs_address'        => str_replace('&', '-', substr($order->bill_address1, 0, 20)), // maximum of 20 characters per spec
		'ssl_address2'           => str_replace('&', '-', substr($order->bill_address2, 0, 20)),
		'ssl_city'               => $order->bill_city_town,
		'ssl_state'              => $order->bill_state_province,
		'ssl_avs_zip'            => preg_replace("/[^A-Za-z0-9]/", "", $order->bill_postal_code),
		'ssl_country'            => $order->bill_country_code,
		'ssl_phone'              => $order->bill_telephone,
		'ssl_email'              => $order->bill_email,
		'ssl_ship_to_company'    => $order->ship_primary_name,
		'ssl_ship_to_first_name' => $order->ship_first_name,
		'ssl_ship_to_last_name'  => $order->ship_last_name,
		'ssl_ship_to_address1'   => $order->ship_address1,
		'ssl_ship_to_address2'   => $order->ship_address2,
		'ssl_ship_to_city'       => $order->ship_city_town,
		'ssl_ship_to_state'      => $order->ship_state_province,
		'ssl_ship_to_zip'        => preg_replace("/[^A-Za-z0-9]/", "", $order->ship_postal_code),
		'ssl_ship_to_country'    => $order->ship_country_code,
		'ssl_ship_to_phone'      => $order->ship_telephone,
/* The following are not used at this time
		'ssl_email_header' => 'Not Used',
		'ssl_email_apprvl_header_html => 'Not Used',
		'ssl_email_decl_header_html => 'Not Used',
		'ssl_email_footer => 'Not Used',
		'ssl_email_apprvl_footer_html => 'Not Used',
		'ssl_email_decl_footer_html => 'Not Used',
		'ssl_do_customer_email => 'FALSE', // set up in admin on Virtual Merchant website
		'ssl_do_merchant_email => 'FALSE', // set up in admin on Virtual Merchant website
		'ssl_merchant_email' => COMPANY_EMAIL,
		'ssl_header_color' => '',
		'ssl_text_color' => '',
		'ssl_background_color' => '',
		'ssl_table_color' => '',
		'ssl_link_color' => '',
*/
		'ssl_show_form'          => 'FALSE',
/* The following are ignored when ssl_show_form = FALSE
		'ssl_header_html' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_footer_html' => 'Not Used', // set up in admin on Virtual Merchant website
*/
		'ssl_result_format'      => 'ASCII'
/* The following are ignored when ssl_result_format = ASCII
		'ssl_receipt_header_html' => 'Not Used', 		// set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_header_html' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_header_html' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_footer_html' => 'Not Used', 		// set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_footer_html' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_footer_html' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_link_method' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_method' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_method' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_link_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_post_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_post_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_get_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_get_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_error_url' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_link_text' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_apprvl_text' => 'Not Used', // set up in admin on Virtual Merchant website
		'ssl_receipt_decl_text' => 'Not Used', // set up in admin on Virtual Merchant website
*/
    );

    if (MODULE_PAYMENT_NOVA_XML_TESTMODE == 'Test') {
      $submit_data['ssl_test_mode'] = 'TRUE';
    }

    // concatenate the submission data and put into $data variable
	$data = '?xmldata=<txn>'; // initiate XML string
    while(list($key, $value) = each($submit_data)) {
    	if ($value <> '') $data .= '<' . urlencode($key) . '>' . urlencode($value) . '</' . urlencode($key) . '>';
    }
	$data .= '</txn>'; // terminate XML string

    // SEND DATA BY CURL SECTION
    // Post order info data to Nova, make sure you have cURL support installed
    $url = 'https://www.myvirtualmerchant.com/VirtualMerchant/processxml.do' . $data;
	$GetPost = 'GET';	
//echo 'transmit url = '; echo htmlspecialchars($url); echo '<br /><br />';

    $ch = curl_init();

	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_TIMEOUT, 10); // times out after 10 seconds 
//	curl_setopt($ch, CURLOPT_HEADER, 0);
//	curl_setopt($ch, CURLOPT_VERBOSE, 0);
	if ($GetPost == 'POST') {
	  curl_setopt($ch, CURLOPT_POST, 1);
	  curl_setopt($ch, CURLOPT_POSTFIELDS, '');
    }
    if (CURL_PROXY_REQUIRED == 'True') {
      curl_setopt ($ch, CURLOPT_HTTPPROXYTUNNEL, true);
      curl_setopt ($ch, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
      curl_setopt ($ch, CURLOPT_PROXY, CURL_PROXY_SERVER_DETAILS);
    }
	$authorize = curl_exec($ch); 
	// Check for curl errors
	$curlerrornum = curl_errno($ch);
	$curlerror = curl_error($ch);
	curl_close ($ch);
	if ($curlerrornum) { 
		$messageStack->add('XML Read Error (cURL) #' . $curlerrornum . '. Description = ' . $curlerror,'error');
		return true;
	}
//echo 'response xml = ' . htmlspecialchars($authorize) . '<br />';

	// since the response is only one level deep, we can do a simple parse
	$authorize = trim($authorize);
	$authorize = substr($authorize, strpos($authorize, '<txn>') + 5); // remove up to and including the container tag
	$authorize = substr($authorize, 0, strpos($authorize, '</txn>')); // remove from end container tag on
	$results = array();
	$runaway_loop_counter = 0;
	while ($authorize) {
		$key = substr($authorize, strpos($authorize, '<') + 1, strpos($authorize, '>') - 1);
		$authorize = substr($authorize, strpos($authorize, '>') + 1); // remove start tag
		$value = substr($authorize, 0, strpos($authorize, '<')); // until start of end tag
		$authorize = substr($authorize, strpos($authorize, '>') + 1); // remove end tag
		$results[$key] = $value;
		if ($runaway_loop_counter++ > 1000) break;
	}

    $this->transaction_id = $results['ssl_txn_id'];
    $this->auth_code = $results['ssl_approval_code'];

    // If the response code is not 0 (approved) then redirect back to the payment page with the appropriate error message
    if ($results['errorCode']) {
      $messageStack->add('Decline Code #' . $results['errorCode'] . ': ' . $results['errorMessage'] . ' - ' . MODULE_PAYMENT_NOVA_XML_TEXT_DECLINED_MESSAGE, 'error');
	  return true;
    } elseif ($results['ssl_result'] == '0') {
		$messageStack->add($results['ssl_result_message'] . ' - Approval code: ' . $this->auth_code . ' --> CVV2 results: ' . $this->cvv_codes[$results['ssl_cvv2_response']], 'success');
		$messageStack->add('Address verification results: ' . $this->avs_codes[$results['ssl_avs_response']], 'success');
		return false;
	}
	$messageStack->add($results['ssl_result_message'] . ' - ' . MODULE_PAYMENT_NOVA_XML_TEXT_DECLINED_MESSAGE, 'error');
	return true;
  }
  /**
   * Post-process activities.
   *
   * @return boolean
   */
  function after_process() {
    return false;
  }
  /**
   * Check to see whether module is installed
   *
   * @return boolean
   */
  function check() {
    global $db;
    if (!isset($this->_check)) {
      $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_NOVA_XML_STATUS'");
      $this->_check = $check_query->RecordCount();
    }
    return $this->_check;
  }
  /**
   * Install the payment module and its configuration settings
   *
   */
  function install() {
    global $db;
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable Nova Network Module', 'MODULE_PAYMENT_NOVA_XML_STATUS', 'True', 'Do you want to accept Nova Network payments ?', '6', '0', 'cfg_select_option(array(\'True\', \'False\'), ', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Merchant ID', 'MODULE_PAYMENT_NOVA_XML_MERCHANT_ID', '', 'The Nova Merchant ID used for the Nova Network service:', '6', '0', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('User ID', 'MODULE_PAYMENT_NOVA_XML_USER_ID', '', 'The Nova User ID used for the Nova Network service:', '6', '0', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('PIN', 'MODULE_PAYMENT_NOVA_XML_PIN', '', 'PIN used for the Nova Network service:', '6', '0', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Transaction Mode', 'MODULE_PAYMENT_NOVA_XML_TESTMODE', 'Test', 'Transaction mode used for processing orders', '6', '0', 'cfg_select_option(array(\'Test\', \'Production\'), ', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Authorization Type', 'MODULE_PAYMENT_NOVA_XML_AUTHORIZATION_TYPE', 'Authorize', 'Do you want submitted credit card transactions to be authorized only, or authorized and captured?', '6', '0', 'cfg_select_option(array(\'Authorize\', \'Authorize/Capture\'), ', now())");
//    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Store the Credit Card Number', 'MODULE_PAYMENT_NOVA_XML_STORE_NUMBER', 'False', 'Do you want to store the Credit Card Number. Security Note: The Credit Card Number will be stored unenecrypted.', '6', '0', 'cfg_select_option(array(\'True\', \'False\'), ', now())");
    $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_NOVA_XML_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '0', now())");
  }
  /**
   * Remove the module and all its settings
   *
   */
  function remove() {
    global $db;
    $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
  }
  /**
   * Internal list of configuration keys used for configuration of the module
   *
   * @return array
   */
  function keys() {
    return array(
	  'MODULE_PAYMENT_NOVA_XML_STATUS', 
	  'MODULE_PAYMENT_NOVA_XML_MERCHANT_ID', 
	  'MODULE_PAYMENT_NOVA_XML_USER_ID', 
	  'MODULE_PAYMENT_NOVA_XML_PIN', 
	  'MODULE_PAYMENT_NOVA_XML_TESTMODE', 
	  'MODULE_PAYMENT_NOVA_XML_AUTHORIZATION_TYPE', 
//	  'MODULE_PAYMENT_NOVA_XML_STORE_NUMBER', 
	  'MODULE_PAYMENT_NOVA_XML_SORT_ORDER' 
	);
  }
}

?>